@@ -24,6 +24,8 @@
 #include <arpa/inet.h>
 #include <time.h>
 #include <inttypes.h>
+#include <sys/stat.h>
+#include <sys/types.h>
 
 #include <osmocom/core/talloc.h>
 #include <osmocom/core/utils.h>
@@ -39,6 +41,7 @@
 #include <osmocom/sgsn/gprs_gmm.h>
 #include <osmocom/sgsn/gprs_bssgp.h>
 #include <osmocom/sgsn/mmctx.h>
+#include <osmocom/sgsn/gprs_routing_area.h>
 #include <osmocom/sgsn/gtp_ggsn.h>
 #include <osmocom/sgsn/gtp_mme.h>
 #include <osmocom/sgsn/vty.h>
@@ -404,6 +407,11 @@ DEFUN(cfg_sgsn_state_dir, cfg_sgsn_state_dir_cmd,
 	"Set the directory for the GTP State file\n"
 	"Local Directory\n")
 {
+	if (mkdir(argv[0], 0755) == -1 && errno != EEXIST) {
+		vty_out(vty, "%% Failed to create state-dir: %s%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
+
 	osmo_talloc_replace_string(sgsn, &sgsn->cfg.gtp_statedir, argv[0]);
 
 	return CMD_SUCCESS;
@@ -644,7 +652,7 @@ static void vty_dump_mmctx(struct vty *vty, const char *pfx,
 		pfx, mm->msisdn, id, mm->hlr, VTY_NEWLINE);
 	vty_out(vty, "%s  GMM State: %s, Routeing Area: %s, Cell ID: %u%s",
 		pfx, osmo_fsm_inst_state_name(mm->gmm_fsm),
-		osmo_rai_name(&mm->ra), mm->gb.cell_id, VTY_NEWLINE);
+		osmo_rai_name2(&mm->ra), mm->gb.cell_id, VTY_NEWLINE);
 	vty_out(vty, "%s  MM State: %s, RAN Type: %s%s", pfx, mm_state_name,
 		get_value_string(sgsn_ran_type_names, mm->ran_type), VTY_NEWLINE);
 
@@ -990,8 +998,17 @@ static void subscr_dump_full_vty(struct vty *vty, struct gprs_subscr *gsub, int
 	}
 
 	llist_for_each_entry(pdp, &gsub->sgsn_data->pdp_list, list) {
-		vty_out(vty, "    PDP info: Id: %d, Type: 0x%04x, APN: '%s'",
-			pdp->context_id, pdp->pdp_type, pdp->apn_str);
+		char ip_str[INET6_ADDRSTRLEN] = { 0 };
+
+		vty_out(vty, "    PDP info: Id: %d, Addr(Org: 0x%02x Type: 0x%02x",
+			pdp->context_id, pdp->pdp_type_org, pdp->pdp_type_nr);
+
+		if (pdp->pdp_address[0].u.sa.sa_family != AF_UNSPEC)
+			vty_out(vty, " Addr[0]: %s", osmo_sockaddr_ntop(&pdp->pdp_address[0].u.sa, ip_str));
+		if (pdp->pdp_address[0].u.sa.sa_family != AF_UNSPEC)
+			vty_out(vty, " Addr[1]: %s", osmo_sockaddr_ntop(&pdp->pdp_address[1].u.sa, ip_str));
+
+		vty_out(vty, ") APN: '%s'", pdp->apn_str);
 
 		if (pdp->qos_subscribed_len)
 			vty_out(vty, " QoS: %s", osmo_hexdump(pdp->qos_subscribed, pdp->qos_subscribed_len));
@@ -1317,7 +1334,7 @@ DEFUN(page_subscr, page_subscr_info_cmd,
 		return CMD_WARNING;
 	}
 
-	sgsn_bssgp_page_ps_ra(mm);
+	sgsn_ra_geran_page_ra(&mm->ra, mm);
 	return CMD_SUCCESS;
 }
 
@@ -1859,9 +1876,6 @@ int sgsn_parse_config(const char *config_file)
 	/* make sure sgsn_vty_init() was called before this */
 	OSMO_ASSERT(g_cfg);
 
-	g_cfg->gea_encryption_mask = 0x1; /* support GEA0 by default unless specific encryption config exists */
-	g_cfg->uea_encryption_mask = (1 << OSMO_UTRAN_UEA0); /* support UEA0 by default unless specific encryption config exists */
-
 	rc = vty_read_config_file(config_file, NULL);
 	if (rc < 0) {
 		fprintf(stderr, "Failed to parse the config file: '%s'\n", config_file);
